@@ -584,6 +584,8 @@ def get_signature(fn, name=None):
     if sig.return_annotation is not inspect._empty:
         if isinstance(returns, str):
             signature.outputs[returns] = TensorSpec(TensorShape([None]), optional=False, name=returns)
+        elif returns is Tensor:
+            pass
         else:
             for i in range(len(returns)):
                 signature.outputs['output_{0}'.format(i)] = TensorSpec(TensorShape([None]), optional=False, name='output_{0}'.format(i))
