@@ -25,12 +25,13 @@ from trident.data.bbox_common import *
 
 if get_backend() == 'pytorch':
     from trident.backend.pytorch_backend import try_map_args_and_call, Layer
-    from trident.backend.pytorch_ops import to_numpy, to_tensor, arange, shuffle, cast, clip, sqrt, int_shape, argmax, softmax, any_abnormal_number, reduce_any, ndim, exp,concate, \
+    from trident.backend.pytorch_ops import to_numpy, to_tensor, arange, shuffle, cast, clip, sqrt, int_shape, argmax, softmax, any_abnormal_number, reduce_any, ndim, exp, \
+        concate, \
         expand_dims
 
 elif get_backend() == 'tensorflow':
     from trident.backend.tensorflow_backend import try_map_args_and_call, Layer
-    from trident.backend.tensorflow_ops import to_numpy, to_tensor, arange, shuffle, cast, clip, sqrt, int_shape, concate, zeros_like, ones_like, argmax, softmax,concate, \
+    from trident.backend.tensorflow_ops import to_numpy, to_tensor, arange, shuffle, cast, clip, sqrt, int_shape, concate, zeros_like, ones_like, argmax, softmax, concate, \
         any_abnormal_number, ndim, exp, not_equal, reduce_any, expand_dims
 
 if is_in_ipython() or is_in_colab():
@@ -44,15 +45,15 @@ __all__ = ['VisualizationCallbackBase', 'TileImageCallback', 'PrintGradientsCall
 
 
 class VisualizationCallbackBase(CallbackBase):
-    def __init__(self, frequency=-1, unit='batch',  save_path: str = None, imshow=False):
+    def __init__(self, frequency=-1, unit='batch', save_path: str = None, imshow=False):
         super(VisualizationCallbackBase, self).__init__()
         self.is_in_ipython = is_in_ipython()
         self.is_in_colab = is_in_colab()
         self.frequency = frequency
-        if unit in ('batch','step','epoch'):
+        if unit in ('batch', 'step', 'epoch'):
             self.unit = unit
         else:
-            print(red_color('Only [batch, step, epoch] are valid unit.',True))
+            print(red_color('Only [batch, step, epoch] are valid unit.', True))
         if save_path is None:
             save_path = 'results'
         self.save_path = make_dir_if_need(save_path)
@@ -62,7 +63,7 @@ class VisualizationCallbackBase(CallbackBase):
 
 
 class TileImageCallback(VisualizationCallbackBase):
-    def __init__(self, frequency=-1, unit='batch',  save_path: str = 'results',
+    def __init__(self, frequency=-1, unit='batch', save_path: str = 'results',
                  name_prefix: str = 'tile_image_{0}.png', row=3, include_input=True, include_output=True, include_target=True,
                  include_mask=None, reverse_image_transform=None, imshow=False):
         super(TileImageCallback, self).__init__(frequency, unit, save_path, imshow)
@@ -146,19 +147,19 @@ class TileImageCallback(VisualizationCallbackBase):
             if len(reverse_image_transform_funcs) > 1:
                 output_arr = []
                 for i in range(len(output)):
-                    out=reverse_image_transform(output[i])
-                    output_arr.append(np.clip(out,0,255))
+                    out = reverse_image_transform(output[i])
+                    output_arr.append(np.clip(out, 0, 255))
                 tile_images_list.append(output_arr)
             else:
                 output_arr = to_numpy(output).transpose([0, 2, 3, 1]) if get_backend() != 'tensorflow' else to_numpy(output)
 
                 if output_arr.max() <= 1.2 and output_arr.min() >= 0:
-                    tile_images_list.append(np.clip(output_arr * 255,0,255))
+                    tile_images_list.append(np.clip(output_arr * 255, 0, 255))
                 elif output_arr.max() <= 1.2 and output_arr.min() >= -1.2:
 
-                    tile_images_list.append(np.clip(output_arr * 128 + 127,0,255))
+                    tile_images_list.append(np.clip(output_arr * 128 + 127, 0, 255))
                 else:
-                    tile_images_list.append(np.clip(output_arr,0,255))
+                    tile_images_list.append(np.clip(output_arr, 0, 255))
 
             legend.append('output')
 
@@ -170,11 +171,12 @@ class TileImageCallback(VisualizationCallbackBase):
         plt.close()
 
     def on_batch_end(self, training_context):
-        if self.frequency>0  and ((self.unit=='batch' and (training_context['current_batch']+1) % self.frequency == 0) or (self.unit=='step' and (training_context['steps']+1) % self.frequency == 0) ):
+        if self.frequency > 0 and ((self.unit == 'batch' and (training_context['current_batch'] + 1) % self.frequency == 0) or (
+                self.unit == 'step' and (training_context['steps'] + 1) % self.frequency == 0)):
             self.plot_tile_image(training_context)
 
     def on_epoch_end(self, training_context):
-        if self.frequency>0  and (self.unit=='epoch' and training_context['current_batch']==0 and  (training_context['current_epoch']+1) % self.frequency == 0):
+        if self.frequency > 0 and (self.unit == 'epoch' and training_context['current_batch'] == 0 and (training_context['current_epoch'] + 1) % self.frequency == 0):
             self.plot_tile_image(training_context)
 
 
@@ -228,7 +230,8 @@ class GanTileImageCallback(VisualizationCallbackBase):
             plt.close()
 
     def on_batch_end(self, training_context):
-        if self.frequency>0  and ((self.unit=='batch' and (training_context['current_batch']+1) % self.frequency == 0) or (self.unit=='step' and (training_context['steps']+1) % self.frequency == 0) or not self.sample_enough ):
+        if self.frequency > 0 and ((self.unit == 'batch' and (training_context['current_batch'] + 1) % self.frequency == 0) or (
+                self.unit == 'step' and (training_context['steps'] + 1) % self.frequency == 0) or not self.sample_enough):
             if self.sample_enough:
                 self.tile_images_list = []
                 self.output_arr = []
@@ -236,7 +239,8 @@ class GanTileImageCallback(VisualizationCallbackBase):
             self.plot_tile_image(training_context)
 
     def on_epoch_end(self, training_context):
-        if self.frequency>0  and ((self.unit=='epoch' and training_context['current_batch']==0 and  (training_context['current_epoch']+1) % self.frequency == 0)or not self.sample_enough):
+        if self.frequency > 0 and (
+                (self.unit == 'epoch' and training_context['current_batch'] == 0 and (training_context['current_epoch'] + 1) % self.frequency == 0) or not self.sample_enough):
             if self.sample_enough:
                 self.tile_images_list = []
                 self.output_arr = []
@@ -245,16 +249,17 @@ class GanTileImageCallback(VisualizationCallbackBase):
 
 
 class SegTileImageCallback(VisualizationCallbackBase):
-    def __init__(self, frequency=-1, unit='batch',  save_path: str = 'results', reverse_image_transform=None,
-                 palette=None, background=(120, 120, 120), name_prefix: str = 'segtile_image_{0}.png', imshow=False):
+    def __init__(self, frequency=-1, unit='batch', save_path: str = 'results', reverse_image_transform=None,
+                 is_label_mask=False, palette=None, background=(120, 120, 120), name_prefix: str = 'segtile_image_{0}.png', imshow=False):
         super(SegTileImageCallback, self).__init__(frequency, unit, save_path, imshow)
         self.is_in_ipython = is_in_ipython()
         self.is_in_colab = is_in_colab()
+        self.is_label_mask = is_label_mask
         self.palette = palette
         self.tile_image_name_prefix = name_prefix
         self.reverse_image_transform = reverse_image_transform
 
-        self.background =to_numpy(background)
+        self.background = to_numpy(background)
 
     def plot_tile_image(self, training_context):
         axis = 1
@@ -262,17 +267,16 @@ class SegTileImageCallback(VisualizationCallbackBase):
             axis = -1
         new_shape = [1] * 4
         new_shape[axis] = 3
-        background =np.reshape(to_numpy(self.background), new_shape)
+        background = np.reshape(to_numpy(self.background), new_shape)
         tile_images_list = []
         input = None
         target = None
         output = None
-        is_label_mask = False
+        is_label_mask = self.is_label_mask
         data_feed = training_context['data_feed']
         data = training_context['train_data']
         model = training_context['current_model']
-        if model.output_shape[model.filter_index] > 2:
-            is_label_mask = True
+
         # if len(data) >= 3:
         for data_key in data.key_list:
             if data_key == data_feed[model.signature.inputs.key_list[0]]:
@@ -310,7 +314,7 @@ class SegTileImageCallback(VisualizationCallbackBase):
         input = to_numpy(input)
         for i in range(len(input)):
             input_arr.append(image_backend_adaption(self.reverse_image_transform(input[i])))
-        input_arr=np.stack(input_arr,axis=0)
+        input_arr = np.stack(input_arr, axis=0)
         # input_arr=np.asarray(input_arr)
         tile_images_list.append(input_arr)
 
@@ -342,7 +346,8 @@ class SegTileImageCallback(VisualizationCallbackBase):
         plt.close()
 
     def on_batch_end(self, training_context):
-        if self.frequency>0  and ((self.unit=='batch' and (training_context['current_batch']+1) % self.frequency == 0) or (self.unit=='step' and (training_context['steps']+1) % self.frequency == 0) ):
+        if self.frequency > 0 and ((self.unit == 'batch' and (training_context['current_batch'] + 1) % self.frequency == 0) or (
+                self.unit == 'step' and (training_context['steps'] + 1) % self.frequency == 0)):
             self.plot_tile_image(training_context)
 
     def on_epoch_end(self, training_context):
@@ -403,11 +408,12 @@ class DetectionPlotImageCallback(VisualizationCallbackBase):
         plt.close()
 
     def on_batch_end(self, training_context):
-        if self.frequency>0  and ((self.unit=='batch' and (training_context['current_batch']+1) % self.frequency == 0) or (self.unit=='step' and (training_context['steps']+1) % self.frequency == 0) ):
+        if self.frequency > 0 and ((self.unit == 'batch' and (training_context['current_batch'] + 1) % self.frequency == 0) or (
+                self.unit == 'step' and (training_context['steps'] + 1) % self.frequency == 0)):
             self.plot_detection_image(training_context)
 
     def on_epoch_end(self, training_context):
-        if self.frequency>0  and (self.unit=='epoch' and training_context['current_batch']==0 and  (training_context['current_epoch']+1) % self.frequency == 0):
+        if self.frequency > 0 and (self.unit == 'epoch' and training_context['current_batch'] == 0 and (training_context['current_epoch'] + 1) % self.frequency == 0):
             self.plot_detection_image(training_context)
 
 
@@ -430,7 +436,7 @@ class PlotLossMetricsCallback(VisualizationCallbackBase):
             self.training_items = training_context['training_items']
 
     def on_overall_batch_end(self, training_context):
-        if not self.is_inplace  and training_context['steps']>10:
+        if not self.is_inplace and training_context['steps'] > 10:
             if self.frequency > 0 and ((self.unit == 'batch' and (training_context['steps'] + 1) % self.frequency == 0) or (
                     self.unit == 'step' and (training_context['steps'] + 1) % self.frequency == 0)):
                 if is_in_ipython() and self.counter == self.clean_ipython_output_frequency:
@@ -489,16 +495,18 @@ class PrintGradientsCallback(VisualizationCallbackBase):
     def __init__(self, frequency=-1, unit='batch'):
         super(PrintGradientsCallback, self).__init__(frequency, unit)
 
-        self.first_layer =OrderedDict()
-        self.last_layer =OrderedDict()
-        self.is_modulefict=False
+        self.first_layer = OrderedDict()
+        self.last_layer = OrderedDict()
+        self.is_modulefict = False
         self.lines = []
 
     def on_optimization_step_start(self, training_context):
         if get_backend() == 'pytorch':
             with torch.no_grad():
-                if self.frequency>0  and ((self.unit=='batch' and (training_context['current_batch']+1) % self.frequency == 0) or (self.unit=='step' and (training_context['steps']+1) % self.frequency == 0) or (self.unit=='epoch' and  (training_context['current_epoch']+1) % self.frequency == 0)):
-                    grad_dict =OrderedDict()
+                if self.frequency > 0 and ((self.unit == 'batch' and (training_context['current_batch'] + 1) % self.frequency == 0) or (
+                        self.unit == 'step' and (training_context['steps'] + 1) % self.frequency == 0) or (
+                                                   self.unit == 'epoch' and (training_context['current_epoch'] + 1) % self.frequency == 0)):
+                    grad_dict = OrderedDict()
                     if 'grads_state' not in training_context:
                         training_context['grads_state'] = OrderedDict()
                     if training_context['current_batch'] == 0 and training_context['current_epoch'] > 0:
@@ -506,55 +514,63 @@ class PrintGradientsCallback(VisualizationCallbackBase):
                         self.first_layer = OrderedDict()
                         self.last_layer = OrderedDict()
 
-                    if len(self.first_layer) ==0 and  len(self.last_layer) == 0:
-                        if  training_context['current_model'][-1].__class__.__name__=='ModuleDict':
-                            self.is_modulefict=True
-                            for k,v in training_context['current_model'][-1].items():
-                                last_layer_name=''
-                                for name,module in v.named_modules():
-                                    if len([pk  for pk,pv in module._parameters.items() if 'bias' not in pk and pv.requires_grad])> 0:
-                                        last_layer_name=module.relative_name
-                                if last_layer_name!='':
-                                    self.last_layer[last_layer_name]=k
+                    if len(self.first_layer) == 0 and len(self.last_layer) == 0:
+                        if training_context['current_model'][-1].__class__.__name__ == 'ModuleDict':
+                            self.is_modulefict = True
+                            for k, v in training_context['current_model'][-1].items():
+                                last_layer_name = ''
+                                for name, module in v.named_modules():
+                                    if len([pk for pk, pv in module._parameters.items() if 'bias' not in pk and pv.requires_grad]) > 0:
+                                        last_layer_name = module.relative_name
+                                if last_layer_name != '':
+                                    self.last_layer[last_layer_name] = k
 
                         first_layer_name = ''
                         last_layer_name = ''
                         for k, v in training_context['current_model'].named_modules():
-                            if len([ pk  for pk,pv in v._parameters.items() if 'bias' not in pk and pv is not None and pv.requires_grad])> 0:
-                                if first_layer_name=='':
-                                    first_layer_name =v.relative_name
+                            if len([pk for pk, pv in v._parameters.items() if 'bias' not in pk and pv is not None and pv.requires_grad]) > 0:
+                                if first_layer_name == '':
+                                    first_layer_name = v.relative_name
                                     self.first_layer[first_layer_name] = 'first_layer'
 
                                 if not self.is_modulefict:
-                                    last_layer_name=v.relative_name
+                                    last_layer_name = v.relative_name
                         if last_layer_name != '' and not self.is_modulefict:
-                            self.last_layer[last_layer_name] =  'last_layer'
-
+                            self.last_layer[last_layer_name] = 'last_layer'
 
                     for name, module in training_context['current_model'].named_modules():
                         if module.relative_name in self.first_layer or module.relative_name in self.last_layer:
-                            grads_data=[np.abs(np.reshape(to_numpy(pv.grad.data),-1))  for pk, pv in module._parameters.items() if 'bias' not in pk and pv is not None and pv.requires_grad and pv.grad is not None]
-                            weights_data = [np.abs(np.reshape(to_numpy(pv.data), -1)) for pk, pv in module._parameters.items() if 'bias' not in pk and pv is not None and pv.requires_grad ]
+                            grads_data = [np.abs(np.reshape(to_numpy(pv.grad.data), -1)) for pk, pv in module._parameters.items() if
+                                          'bias' not in pk and pv is not None and pv.requires_grad and pv.grad is not None]
+                            weights_data = [np.abs(np.reshape(to_numpy(pv.data), -1)) for pk, pv in module._parameters.items() if
+                                            'bias' not in pk and pv is not None and pv.requires_grad]
                             if ctx.enable_tensorboard and ctx.summary_writer is not None:
-                                ctx.summary_writer.add_histogram(training_context['training_name'] + '/gradients/' + self.first_layer[module.relative_name] if module.relative_name in self.first_layer else self.last_layer[module.relative_name], np.concatenate(grads_data,axis=0), training_context['steps'])
-                                ctx.summary_writer.add_histogram(training_context['training_name'] + '/weights/' + self.first_layer[module.relative_name] if module.relative_name in self.first_layer else self.last_layer[module.relative_name], np.concatenate(weights_data,axis=0), training_context['steps'])
-                            if len(grads_data)>0:
-                                grads_data=np.concatenate(grads_data,axis=0).mean()
+                                ctx.summary_writer.add_histogram(
+                                    training_context['training_name'] + '/gradients/' + self.first_layer[module.relative_name] if module.relative_name in self.first_layer else
+                                    self.last_layer[module.relative_name], np.concatenate(grads_data, axis=0), training_context['steps'])
+                                ctx.summary_writer.add_histogram(
+                                    training_context['training_name'] + '/weights/' + self.first_layer[module.relative_name] if module.relative_name in self.first_layer else
+                                    self.last_layer[module.relative_name], np.concatenate(weights_data, axis=0), training_context['steps'])
+                            if len(grads_data) > 0:
+                                grads_data = np.concatenate(grads_data, axis=0).mean()
                             else:
-                                grads_data=None
+                                grads_data = None
                             if module.relative_name in self.first_layer:
-                                training_context['grads_state']['first_layer']=grads_data
+                                training_context['grads_state']['first_layer'] = grads_data
                             elif module.relative_name in self.last_layer:
-                                training_context['grads_state'][self.last_layer[module.relative_name]]=grads_data
+                                training_context['grads_state'][self.last_layer[module.relative_name]] = grads_data
 
                     if len(training_context['grads_state']) > 0:
-                        grads_str=yellow_color('{0:<16s}'.format(training_context['current_model'].name) + '|'.join(['{0} gradients: {1:<8.3e} '.format(k, v) for k,v in training_context['grads_state'].items() if isinstance(v,numbers.Number)]))
-                        self.lines.append(grads_str+'\n')
+                        grads_str = yellow_color('{0:<16s}'.format(training_context['current_model'].name) + '|'.join(
+                            ['{0} gradients: {1:<8.3e} '.format(k, v) for k, v in training_context['grads_state'].items() if isinstance(v, numbers.Number)]))
+                        self.lines.append(grads_str + '\n')
 
 
         elif get_backend() == 'tensorflow':
-            if self.frequency>0  and ((self.unit=='batch' and (training_context['current_batch']+1) % self.frequency == 0) or (self.unit=='step' and (training_context['steps']+1) % self.frequency == 0) or (self.unit=='epoch' and training_context['current_batch']==0 and  (training_context['current_epoch']+1) % self.frequency == 0)):
-                grad_dict =OrderedDict()
+            if self.frequency > 0 and ((self.unit == 'batch' and (training_context['current_batch'] + 1) % self.frequency == 0) or (
+                    self.unit == 'step' and (training_context['steps'] + 1) % self.frequency == 0) or (
+                                               self.unit == 'epoch' and training_context['current_batch'] == 0 and (training_context['current_epoch'] + 1) % self.frequency == 0)):
+                grad_dict = OrderedDict()
                 if 'grads_state' not in training_context:
                     training_context['grads_state'] = OrderedDict()
                 if training_context['current_batch'] == 0 and training_context['current_epoch'] > 0:
@@ -586,13 +602,14 @@ class PrintGradientsCallback(VisualizationCallbackBase):
                         self.last_layer[last_layer_name] = 'last_layer'
 
                 grads_and_vars = list(training_context['optimizer'].grads_and_vars)
-                grads_dict=OrderedDict()
-                for grad, var  in grads_and_vars:
-                    grads_dict[var.ref()]=to_numpy(grad)
+                grads_dict = OrderedDict()
+                for grad, var in grads_and_vars:
+                    grads_dict[var.ref()] = to_numpy(grad)
 
                 for name, module in training_context['current_model'].named_modules():
                     if module.relative_name in self.first_layer or module.relative_name in self.last_layer:
-                        grads_data = [np.abs(np.reshape(grads_dict[pv.ref()], -1)) for pk, pv in module._parameters.items() if  'bias' not in pk and pv.trainable and grads_dict[pv.ref()] is not None]
+                        grads_data = [np.abs(np.reshape(grads_dict[pv.ref()], -1)) for pk, pv in module._parameters.items() if
+                                      'bias' not in pk and pv.trainable and grads_dict[pv.ref()] is not None]
                         weights_data = [np.abs(np.reshape(to_numpy(pv.value()), -1)) for pk, pv in module._parameters.items() if 'bias' not in pk and pv.trainable]
                         if ctx.enable_tensorboard and ctx.summary_writer is not None:
                             ctx.summary_writer.add_histogram(
@@ -611,11 +628,9 @@ class PrintGradientsCallback(VisualizationCallbackBase):
                             training_context['grads_state'][self.last_layer[module.relative_name]] = grads_data
 
                 if len(training_context['grads_state']) > 0:
-                    grad_str=yellow_color('{0:<16s}'.format(training_context['current_model'].name) + '|'.join(
+                    grad_str = yellow_color('{0:<16s}'.format(training_context['current_model'].name) + '|'.join(
                         ['{0} gradients: {1:<8.3e} '.format(k, v) for k, v in training_context['grads_state'].items() if isinstance(v, numbers.Number)]))
-                    self.lines.append(grad_str+'\n')
-
-
+                    self.lines.append(grad_str + '\n')
 
     def on_overall_batch_end(self, training_context):
         if len(self.lines) > 0:
