@@ -1166,24 +1166,28 @@ class Ranger(Optimizer):
                         step_size = 1.0 / (1 - beta1 ** state['step'])
                     buffered[2] = step_size
 
-                if group['weight_decay'] != 0:
-                    p_data_fp32.add_(-group['weight_decay'] * group['lr'], p_data_fp32)
-
-                if self.gradient_centralization in ['all', 'gc']:
-                    if len(list(grad.size())) > 1:
-                        grad.add_(-grad.mean(dim=tuple(range(1, len(list(grad.size())))), keepdim=True))
 
                 if N_sma > self.N_sma_threshhold:
                     denom = exp_avg_sq.sqrt().add_(group['eps'])
-                    p_data_fp32.addcdiv_(-step_size * group['lr'], exp_avg, denom)
+                    #p_data_fp32.addcdiv_(-step_size * group['lr'], exp_avg, denom)
+                    G_grad = exp_avg / denom
                 else:
-                    p_data_fp32.add_(-step_size * group['lr'], exp_avg)
+                    #p_data_fp32.add_(exp_avg, alpha=-step_size * group['lr'])
+                    G_grad = exp_avg
+
+                if group['weight_decay'] != 0:
+                    #p_data_fp32.add_(p_data_fp32, alpha=-group['weight_decay'] * group['lr'])
+                    G_grad.add_(p_data_fp32, alpha=group['weight_decay'])
+
+                if self.gradient_centralization in ['all', 'gc']:
+                    if len(list(grad.size())) > 1:
+                        G_grad.add_(-G_grad.mean(dim=tuple(range(1, len(list(grad.size())))), keepdim=True))
 
                 if any_abnormal_number(p_data_fp32):
                     sys.stderr.write('{0} p_data has abnormal value,trident automatically replace these abnormal value to zero.\n\r'.format(self.__class__.__name__))
                     p_data = where(is_abnormal_number(p_data_fp32), p.data, p_data_fp32)
 
-
+                p_data_fp32.add_(G_grad, alpha=-step_size * group['lr'])
                 p.data.copy_(p_data_fp32)
                 # integrated look ahead...
                 # we do it at the param level instead of group level
@@ -1191,7 +1195,7 @@ class Ranger(Optimizer):
                     # get access to slow param tensor
                     slow_p = state['slow_buffer']
                     # (fast weights - slow weights) * alpha
-                    slow_p.add_(self.alpha, p.data - slow_p)
+                    slow_p.add_(p.data - slow_p,alpha=self.alpha)
                     if any_abnormal_number(slow_p):
                         sys.stderr.write('{0} p_data has abnormal value,trident automatically replace these abnormal value to zero.\n'.format(self.__class__.__name__))
                         slow_p = where(is_abnormal_number(slow_p), p.data, slow_p)
@@ -1955,6 +1959,7 @@ class RangerAdaBelief(Optimizer):
                     if len(list(G_grad.size())) > 1:
                         G_grad.add_(-G_grad.mean(dim=tuple(range(1, len(list(G_grad.size())))), keepdim=True))
 
+
                 p_data_fp32.add_(G_grad, alpha=-step_size * group['lr'])
 
                 p.data.copy_(p_data_fp32)
@@ -2057,7 +2062,7 @@ class DiffGrad(Optimizer):
                 bias_correction2 = 1 - beta2 ** state['step']
 
                 if group['weight_decay'] != 0:
-                    grad.add_(group['weight_decay'], p.data)
+                    grad.add_(p.data,alpha=group['weight_decay'])
 
                 if self.gradient_centralization in ['all', 'gc']:
                     if len(list(grad.size())) > 1:
