@@ -716,7 +716,7 @@ class BCELoss(_ClassificationLoss):
         super().__init__(axis=axis, sample_weight=sample_weight, auto_balance=auto_balance, from_logits=from_logits, ignore_index=ignore_index, cutoff=cutoff,
                          label_smooth=label_smooth, reduction=reduction, enable_ohem=enable_ohem, ohem_ratio=ohem_ratio, name=name)
         self._built = True
-        self.num_classes = 1
+        self.num_classes = None
         self.is_logsoftmax = False
         self.need_target_onehot = True
         self.is_target_onehot = False
@@ -733,6 +733,8 @@ class BCELoss(_ClassificationLoss):
 
         """
         with self._name_scope:
+            if self.is_logsoftmax:
+                output = exp(output)
             loss = binary_cross_entropy(output, target, from_logits=self.from_logits)
             return loss
 
