@@ -57,12 +57,10 @@ class PolicyBase(Model):
         self.replay_unit = replay_unit
         self.name = name
         self.action_strategy = action_strategy
+
         self.train_steps = -1
         self.frame_steps = -1
-        self.state_pool = []
-        self.reward_pool = []
-        self.action_pool = []
-        self.value_pool = []
+        self.rollout = Rollout()
         self.setting_network()
 
     def setting_network(self):
@@ -276,15 +274,14 @@ class PolicyBase(Model):
                     if self.push_into_memory_criteria(state, action, next_state, reward) or done:
                         self.memory.push(state, action, next_state, reward)
                 elif self.replay_unit == 'episode':
-                    self.state_pool.append(state)
-                    self.action_pool.append(action)
-                    self.reward_pool.append(reward)
+                    self.rollout.collect('state', self.frame_steps, state)
+                    self.rollout.collect('action', self.frame_steps, action)
+                    self.rollout.collect('reward', self.frame_steps, reward)
+
                     if done:
                         if self.push_into_memory_criteria(self.state_pool, self.action_pool, None, self.reward_pool):
                             self.memory.push(self.state_pool, self.action_pool, None, self.reward_pool)
-                        self.state_pool = []
-                        self.action_pool = []
-                        self.reward_pool = []
+                        self.rollout.reset()
 
                 complete = self.episode_complete_criteria()
                 # Train on_step_end
@@ -338,7 +335,6 @@ class PolicyBase(Model):
         except Exception as e:
             print(e)
             PrintException()
-        self.do_on_training_end()
 
     def resume(self, num_episodes=3000, **kwargs):
         pass
@@ -420,33 +416,26 @@ def Critic(cls):
 
 def Actor(cls):
     """Actor Wrapper class that provides proxy access to an instance of actor model."""
+
     class Wrapper:
         def __init__(self, **kwargs):
-            self.kwargs=kwargs
+            self.kwargs = kwargs
             self.wrap = cls()
 
-        def __call__(self,inputs: Union[Dict[TensorSpec,np.ndarray],np.ndarray],spec:TensorSpec=None,**kwargs):
+        def __call__(self, inputs: Union[Dict[TensorSpec, np.ndarray], np.ndarray], spec: TensorSpec = None, **kwargs):
             if self.rn % 5 > 0:
-                return self.wrap(inputs,spec=spec,**kwargs)
+                return self.wrap(inputs, spec=spec, **kwargs)
             else:
                 return inputs
 
-
         def __getattr__(self, name):
             return getattr(self.wrap, name)
 
-
     Wrapper.__name__ = Wrapper.__qualname__ = cls.__name__
     Wrapper.__doc__ = cls.__doc__
     return Wrapper
 
 
-
-
-
-
-
-
 class ActorCriticPolicy(MuiltiNetwork):
     """The base class for any RL policy.
     """
@@ -481,7 +470,6 @@ class ActorCriticPolicy(MuiltiNetwork):
         self.setting_network()
         self.train_steps = -1
         self.frame_steps = -1
-        
 
     def setting_network(self):
         if isinstance(self._networks['actor'], Layer):
@@ -617,7 +605,7 @@ class ActorCriticPolicy(MuiltiNetwork):
             for i in range(len(rewards) - 1, -1, -1):
                 gae = delta[i] + m[i] * gae
                 R[i] = gae
-            R=stack(R,axis=0)
+            R = stack(R, axis=0)
             return R
         except Exception as e:
             print(e)
@@ -697,8 +685,8 @@ class ActorCriticPolicy(MuiltiNetwork):
         self.t = 0
         self.current_epoch = 0
         self.train_steps = -1
-        self.current_screen=None
-        self.last_screen = None
+        self.current_screen = None
+        self.prev_screen = None
 
         if hasattr(self.env, 'recording_enabled'):
             self.env.recording_enabled = True
@@ -715,7 +703,7 @@ class ActorCriticPolicy(MuiltiNetwork):
             self.total_rewards = 0
             state = self.get_observation()
             self.current_screen = state
-            self.last_screen =state
+            self.prev_screen = state
             for t in count():
                 self.frame_steps += 1
                 self.current_batch += 1
@@ -724,15 +712,17 @@ class ActorCriticPolicy(MuiltiNetwork):
                 # # Train on_step_start
                 # if training and train_timing == 'on_step_start' and t % train_every_nstep == 0:
                 #     self.training_model(i_episode, t,num_episodes=num_episodes, repeat_train=repeat_train, batch_size=batch_size)
+                if self.current_screen.shape != self.prev_screen.shape:
+                    self.prev_screen = self.current_screen.copy()
 
-                action = self.select_action((2*self.current_screen +self.last_screen)/3.0 , model_only=True)
+                action = self.select_action((2 * self.current_screen + self.prev_screen) / 3.0, model_only=True)
                 observation, reward, done, info = self.get_rewards(action)
 
                 self.total_rewards += reward
 
-                next_state = self.get_observation() if not done else None
-                self.last_screen=self.current_screen
-                self.current_screen=next_state
+                next_state = self.get_observation() if not done else zeros((1, 3, 84, 84))
+                self.prev_screen = self.current_screen
+                self.current_screen = next_state
 
                 if need_render:
                     self.env.render()
@@ -1190,18 +1180,17 @@ class PPOPolicy(ActorCriticPolicy):
     """
 
     def __init__(self, actor: Layer, critic: Layer, env: gym.Env, use_experience_replay=False, memory_length: int = 10000
-                 , gamma=0.9, beta=0.01,use_gae=False, gae_lambda=0.95, normalize_advantages=False,clip_grad_norm=0.5, name='ppo') -> None:
+                 , gamma=0.9, beta=0.01, use_gae=False, gae_lambda=0.95, normalize_advantages=False, clip_grad_norm=0.5, name='ppo') -> None:
         super(PPOPolicy, self).__init__(actor=actor, critic=critic, env=env, action_strategy=ActionStrategy.OnPolicy, gamma=gamma, use_experience_replay=use_experience_replay,
                                         replay_unit='episode',
                                         memory_length=memory_length, name=name)
         self.state = self.env.reset()
         self.memory = Rollout()
-        self.use_gae=use_gae
-        self.normalize_advantages=normalize_advantages
+        self.use_gae = use_gae
+        self.normalize_advantages = normalize_advantages
         self.gae_lambda = gae_lambda
         self.beta = beta
         self.with_grad_clipping(clip_grad_norm)
-        
 
         self.gamma = gamma
 
@@ -1257,21 +1246,21 @@ class PPOPolicy(ActorCriticPolicy):
         try:
             actor_loss = 0
             critic_loss = 0
-            reward_pool = self.rollout.get_running_mean('reward')
-            probs_pool= self.rollout.probs
+            reward_pool = self.rollout.reward
+            probs_pool = self.rollout.probs
             log_prob_pool = self.rollout.log_prob
             state_pool = self.rollout.state
             action_pool = self.rollout.action
             done_pool = self.rollout.done
 
             last_done = done_pool[-1]
-            last_next_value= 0.0 if last_done else self.critic.model(state_pool[-1])[:, action_pool[-1]].item()
+            last_next_value = 0.0 if last_done else self.critic.model(state_pool[-1])[:, action_pool[-1]].item()
             running_add = last_next_value
             R = []
             V = []
-            Ratio1=[]
+            Ratio1 = []
             Ratio2 = []
-            LogProbs=[]
+            LogProbs = []
             # NewBest = []
             # OldBest = []
 
@@ -1281,7 +1270,7 @@ class PPOPolicy(ActorCriticPolicy):
                 value = self.critic.model(state_pool[i])[:, action_pool[i]]
                 V.insert(0, value)
 
-                new_probs = self.actor( state_pool[i])
+                new_probs = self.actor(state_pool[i])
                 new_m = Categorical(probs=new_probs)
                 new_log_prob = new_m.log_prob(to_tensor(action_pool[i]))
                 LogProbs.insert(0, new_log_prob)
@@ -1292,55 +1281,49 @@ class PPOPolicy(ActorCriticPolicy):
                 # NewBest.insert(0, new_best_action)
                 # current_action=action_pool[i]
 
-
                 ratio1 = exp(new_log_prob - log_prob_pool[i])
-                ratio2=clip(ratio1, 1.0 -  0.2, 1.0 +  0.2)
+                ratio2 = clip(ratio1, 1.0 - 0.2, 1.0 + 0.2)
                 Ratio1.insert(0, ratio1)
                 Ratio2.insert(0, ratio2)
 
+            R = to_tensor(R).detach()
 
-
-
-            R=to_tensor(R).detach()
-            
             V_ = V.copy()[1:]
             V_.append(to_tensor(running_add))
             V = concate(V, axis=0)
             V_ = concate(V_, axis=0)
-            clipped_value=where(V>V_*1.2,V_*1.2,V)
-            clipped_value = where(clipped_value< V_*0.8, V_*0.8, clipped_value)
+            clipped_value = where(V > V_ * 1.2, V_ * 1.2, V)
+            clipped_value = where(clipped_value < V_ * 0.8, V_ * 0.8, clipped_value)
 
-
-            done_pool=to_tensor(done_pool).float().detach()
-            advantages=None
+            done_pool = to_tensor(done_pool).float().detach()
+            advantages = None
             if self.use_gae:
 
                 gaes = self.calculate_gae(V, V_, R, done_pool, self.gamma, self.gae_lambda)
-                clipped_gaes= self.calculate_gae(clipped_value, V_, R, done_pool, self.gamma, self.gae_lambda)
-                advantages = max(gaes,clipped_gaes)
+                clipped_gaes = self.calculate_gae(clipped_value, V_, R, done_pool, self.gamma, self.gae_lambda)
+                advantages = max(gaes, clipped_gaes)
                 if self.normalize_advantages:
-                    advantages=advantages/(advantages.std()+1e-5)
+                    advantages = advantages / (advantages.std() + 1e-5)
             else:
-                advantages = max(V-R,clipped_value-R)
+                advantages = max(V - R, clipped_value - R)
 
             critic_loss = (advantages ** 2).mean()
-            LogProbs= concate(LogProbs, axis=0)
+            LogProbs = concate(LogProbs, axis=0)
 
-            Ratio1= stack(Ratio1, axis=0)
+            Ratio1 = stack(Ratio1, axis=0)
             Ratio2 = stack(Ratio2, axis=0)
             self.actor.batch_metric_history.collect('ratio1', self.train_steps, Ratio1.copy().detach().mean())
             self.actor.batch_metric_history.collect('ratio2', self.train_steps, Ratio2.copy().detach().mean())
 
             surr1 = Ratio1 * (advantages.detach())
-            surr2 = Ratio2* (advantages.detach())
-            actor_loss=None
+            surr2 = Ratio2 * (advantages.detach())
+            actor_loss = None
             if last_done:
-                actor_loss=(-1*LogProbs*advantages.detach()).mean()
+                actor_loss = (-1 * LogProbs * advantages.detach()).mean()
             else:
-                actor_loss = (-1*min(surr1,surr2)).mean()
+                actor_loss = (-1 * min(surr1, surr2)).mean()
 
-            
-            return critic_loss, actor_loss, advantages,R,V
+            return critic_loss, actor_loss, advantages, R, V
         except Exception as e:
             print(e)
 
@@ -1360,7 +1343,7 @@ class PPOPolicy(ActorCriticPolicy):
         state_pool = self.rollout.state
         action_pool = self.rollout.action
         done_pool = self.rollout.done
-        reward_pool = self.rollout.reward
+        reward_pool = self.rollout.get_running_mean('reward')
         gae = 0.0
         R = []
         V = []
@@ -1371,16 +1354,15 @@ class PPOPolicy(ActorCriticPolicy):
                 actor_loss = to_tensor(0.0)
                 critic_loss = to_tensor(0.0)
 
-
-                if (len(reward_pool) <= 3 or self.done == True or random.random() < 0.1) and len(self.memory) > 5 and self.current_epoch>5:
+                if (len(reward_pool) <= 3 or self.done == True or random.random() < 0.1) and len(self.memory) > 5 and self.current_epoch > 5:
                     for i in range(8 if done else 2):
                         try:
                             self.train_steps += 1
                             self.current_batch += 1
                             self.steps = self.train_steps
-                            self.done=False
+                            self.done = False
 
-                            #self.do_on_batch_start()
+                            # self.do_on_batch_start()
 
                             batch_data = self.memory.get_samples(builtins.min(64 if done else 32, len(self.memory)))
                             batch_return = batch_data['return']
@@ -1389,9 +1371,9 @@ class PPOPolicy(ActorCriticPolicy):
                             batch_log_prob = batch_data['log_prob']
 
                             batch_value = concate([self.critic(to_tensor(batch_state[i]))[:, batch_action[i]] for i in range(len(batch_state))], axis=0)
-                            batch_return= to_tensor(batch_return)
+                            batch_return = to_tensor(batch_return)
                             batch_advantages = (batch_return.detach() - batch_value)
-                            
+
                             batch_critic_loss = (batch_advantages ** 2).mean()
                             self.critic.current_loss = batch_critic_loss
                             self.critic.batch_loss_history.collect('critic_loss', self.train_steps, batch_critic_loss.copy().detach())
@@ -1400,7 +1382,6 @@ class PPOPolicy(ActorCriticPolicy):
                             self.critic.current_loss = 0.0
                             batch_actor_loss = to_tensor(0.0)
                             for j in range(len(batch_state)):
-
                                 old_batch_probs = self.old_actor(to_tensor(batch_state[j]))
                                 old_m = Categorical(probs=old_batch_probs)
                                 old_log_prob = old_m.log_prob(to_tensor(batch_action[j]).long())
@@ -1409,7 +1390,7 @@ class PPOPolicy(ActorCriticPolicy):
                                 new_log_prob = new_m.log_prob(to_tensor(batch_action[j]).long())
 
                                 # entropy_loss = entropy_loss + new_m.entropy().mean()
-                                ratio = exp(old_batch_probs -old_batch_probs)
+                                ratio = exp(old_batch_probs - old_batch_probs)
                                 cliprange = 0.2
                                 surr1 = ratio * (batch_advantages[j].detach())
                                 surr2 = clip(ratio, 1.0 - cliprange, 1.0 + cliprange) * (batch_advantages[j].detach())
@@ -1420,12 +1401,12 @@ class PPOPolicy(ActorCriticPolicy):
 
                             self.update_actor(self.current_epoch, self.train_steps, self.total_epoch, 1, 'on_step_end', False, accumulate_grads=False)
                             self.actor.current_loss = 0.0
-                            #self.do_on_batch_end()
+
                         except Exception as e:
                             print(e)
-                            #self.do_on_batch_end()
-                    #self.critic.print_batch_progress(8 if done else 2)
-                    #self.actor.print_batch_progress(8 if done else 2)
+
+                    # self.critic.print_batch_progress(8 if done else 2)
+                    # self.actor.print_batch_progress(8 if done else 2)
 
                 if len(reward_pool) > 3:
 
@@ -1435,10 +1416,8 @@ class PPOPolicy(ActorCriticPolicy):
                         self.steps = self.train_steps
                         self.actor.steps = self.train_steps
                         self.critic.steps = self.train_steps
-                        self.done=done
-                        #self.do_on_batch_start()
-
-
+                        self.done = done
+                        # self.do_on_batch_start()
 
                         # gae = 0
 
@@ -1481,13 +1460,13 @@ class PPOPolicy(ActorCriticPolicy):
                         # # advantages=(advantages-advantages.mean())/(advantages.std()+1e-7)
                         # critic_loss = (advantages ** 2).mean()
 
-                        critic_loss, actor_loss ,advantages, R,V= self.calculate_discounted_returns()
+                        critic_loss, actor_loss, advantages, R, V = self.calculate_discounted_returns()
                         self.critic.current_loss = critic_loss
                         self.critic.batch_loss_history.collect('critic_loss', self.train_steps, critic_loss.copy().detach())
                         self.critic.batch_metric_history.collect('critic_rmse', self.train_steps, critic_loss.copy().detach().sqrt())
 
                         self.update_critic(self.current_epoch, self.train_steps, self.total_epoch, 1, 'on_step_end', self.done, accumulate_grads=False)
-                        #self.critic.print_batch_progress(1)
+                        # self.critic.print_batch_progress(1)
                         self.critic.current_loss = 0
 
                         # keep = []
@@ -1508,9 +1487,9 @@ class PPOPolicy(ActorCriticPolicy):
                             self.memory.collect('action', action_pool[idx])
                             self.memory.collect('log_prob', log_prob_pool[idx])
 
-                        if len(self.memory.get('return'))>3000:
+                        if len(self.memory.get('return')) > 3000:
                             for k in self.memory.keys():
-                                self.memory[k]=self.memory[k][1000:]
+                                self.memory[k] = self.memory[k][1000:]
                         # print('tota; length:',len(self.memory.value_list[0]),keep)
 
                         # print(list(self.critic.model[-1].named_parameters()))
@@ -1531,13 +1510,12 @@ class PPOPolicy(ActorCriticPolicy):
                         #
 
                         self.actor.current_loss = actor_loss
-                        self.actor.batch_loss_history.collect('actor_loss', self.train_steps, actor_loss.copy().detach() )
+                        self.actor.batch_loss_history.collect('actor_loss', self.train_steps, actor_loss.copy().detach())
                         # self.actor.batch_loss_history.collect('entropy_loss', self.train_steps, (-0.01 * entropy_loss).copy().detach() / (len(state_pool) - 1))
 
-
                         self.update_actor(self.current_epoch, self.train_steps, self.total_epoch, 1, 'on_step_end', self.done, accumulate_grads=False)
                         self.actor.current_loss = 0.0
-                        #self.actor.print_batch_progress(1)
+                        # self.actor.print_batch_progress(1)
 
                         self.save_or_sync_weights()
 
@@ -1550,9 +1528,9 @@ class PPOPolicy(ActorCriticPolicy):
                         del R
                         del V
                         gc.collect()
-                        #self.do_on_batch_end()
+
                     except Exception as e:
                         print(e)
-                        #self.do_on_batch_end()
+
             except Exception as e:
                 print(e)
